@@ -1,6 +1,6 @@
 /*
     avicore
-    copyright (c) 2002 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 2002-2003 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,16 +25,12 @@
 #define __CHUNK_H__
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "common.h"
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
 #endif
 
 
-#include <sys/types.h>
-#include <unistd.h>
-#include <glib.h>
-
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -47,7 +43,7 @@ typedef struct ChunkList_Tag {
 	guint32 form;				/* եॿ */
 } ChunkList;
 typedef struct Chunk_Tag {
-	int num;					/* ꥹȤο */
+	gint num;					/* ꥹȤο */
 	ChunkList *list;
 } Chunk;
 
@@ -56,10 +52,10 @@ typedef struct Chunk_Tag {
 	 c0,1ʸ
 	 c1,2ʸ
 	RET,ID																	*/
-#define make2cc(c0,c1)\
-		(\
-			(guint16)(guint8)(c0)|((guint16)(guint8)(c1)<<8)				\
-		)
+#define make2cc(c0,c1)														\
+	(																		\
+		(guint16)(guint8)(c0)|((guint16)(guint8)(c1)<<8)					\
+	)
 
 
 /*	4ĤʸID
@@ -68,17 +64,60 @@ typedef struct Chunk_Tag {
 	 c2,3ʸ
 	 c3,4ʸ
 	RET,ID																	*/
-#define make4cc(c0,c1,c2,c3)\
-		(\
-			(guint32)(guint8)(c0)|((guint32)(guint8)(c1)<<8)				\
-			|((guint32)(guint8)(c2)<<16)|((guint32)(guint8)(c3)<<24)		\
-		)
+#define make4cc(c0,c1,c2,c3)												\
+	(																		\
+		(guint32)(guint8)(c0)|((guint32)(guint8)(c1)<<8)					\
+		|((guint32)(guint8)(c2)<<16)|((guint32)(guint8)(c3)<<24)			\
+	)
+
+
+/*	4ʸɤʸ
+	fcc,4ʸ
+	  n,ʸ(0...3)
+	RET,ʸ																*/
+#define get4cc(fcc,n) ((guint8)(fcc)>>((n)*8))
+
+
+/*	4ʸɤʸˤ
+	fcc,4ʸ
+	RET,4ʸ															*/
+#define tolower4cc(fcc)														\
+	make4cc(																\
+		g_ascii_tolower(get4cc(fcc,0)),										\
+		g_ascii_tolower(get4cc(fcc,1)),										\
+		g_ascii_tolower(get4cc(fcc,2)),										\
+		g_ascii_tolower(get4cc(fcc,3))										\
+	)
+
+
+#define ICTYPE_VIDEO			make4cc('v','i','d','c')
+#define ICTYPE_AUDIO			make4cc('a','u','d','c')
+#define FOURCC_RIFF				make4cc('R','I','F','F')
+#define FOURCC_LIST				make4cc('L','I','S','T')
+#define streamtypeVIDEO			make4cc('v','i','d','s')
+#define streamtypeAUDIO			make4cc('a','u','d','s')
+#define comptypeDIB				make4cc('D','I','B',' ')
+#define formtypeAVI				make4cc('A','V','I',' ')
+#define formtypeWAVE			make4cc('W','A','V','E')
+#define listtypeAVIHEADER		make4cc('h','d','r','l')
+#define listtypeSTREAMHEADER	make4cc('s','t','r','l')
+#define listtypeAVIMOVIE		make4cc('m','o','v','i')
+#define ckidAVIMAINHDR			make4cc('a','v','i','h')
+#define ckidSTREAMHEADER		make4cc('s','t','r','h')
+#define ckidSTREAMFORMAT		make4cc('s','t','r','f')
+#define ckidSTREAMHANDLERDATA	make4cc('s','t','r','d')
+#define ckidSTREAMNAME			make4cc('s','t','r','n')
+#define ckidAVINEWINDEX			make4cc('i','d','x','1')
+#define ckidAVIPADDING			make4cc('J','U','N','K')
+#define ckidWAVEFORMAT			make4cc('f','m','t',' ')
+#define ckidWAVEDATA			make4cc('d','a','t','a')
 
 
 /*	եǥ꥿󥯹¤Τ
 	 fd,եǥ꥿
-	RET,󥯹¤														*/
-Chunk *chunk_open(int fd);
+	RET,󥯹¤,NULL:顼											*/
+#define chunk_open(fd) (lseek(fd,0,SEEK_SET)!=(off_t)-1						\
+												?g_malloc0(sizeof(Chunk)):NULL)
 
 
 /*	󥯹¤Τ
@@ -109,19 +148,19 @@ gboolean chunk_next(int fd,Chunk *ck);
 
 /*	󥯤ID
 	 fd,եǥ꥿
-	RET,ID																	*/
+	RET,ID,-1:顼														*/
 guint32 chunk_id(int fd);
 
 
 /*	󥯤Υ
 	 fd,եǥ꥿
-	RET,ե															*/
+	RET,,-1:顼													*/
 guint32 chunk_size(int fd);
 
 
 /*	󥯤Υե
 	 fd,եǥ꥿
-	RET,ե															*/
+	RET,ե,-1:顼													*/
 guint32 chunk_form(int fd);
 
 
