@@ -20,10 +20,18 @@
 #define __VERSION_H__
 
 
-#include "export.h"
+#include "pe_image.h"
+
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 
 SymbolInfo *version_get_export_symbols(void);
 
 
+#ifdef __cplusplus
+}
+#endif
 #endif
