@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2002 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2003 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#include <ctype.h>
-#include <string.h>
 #include "argument.h"
 #include "misc/misc.h"
 
@@ -53,10 +51,9 @@ gboolean *arg_analyse(gchar **argv,const gboolean boot)
 	{"help",   '?',NULL,          N_("Help")},
 	{NULL,     0,  NULL,          NULL}};
 
-	for (i=0;argv[i]!=NULL;i++);
-	arg_files=g_malloc(i*sizeof(gboolean));
-	memset(arg_files,0,i*sizeof(gboolean));
-	i=0;/* i:Υץ,j:iʸ,k:iʹߤarg */
+	for (i=1;argv[i]!=NULL;i++);
+	arg_files=g_malloc0(i*sizeof(gboolean));
+	i=1;/* i:Υץ,j:iʸ,k:iʹߤarg */
 	while (argv[i]!=NULL)
 		if ((argv[i])[0]=='-') {/* ץ */
 			j=1;
@@ -65,7 +62,7 @@ gboolean *arg_analyse(gchar **argv,const gboolean boot)
 				if ((argv[i])[1]=='-') {	/* Ĺץ */
 					opt=0;
 					for (l=0;arg_opts[l].name!=NULL;l++) {
-						leng=strlen(arg_opts[l].name);
+						leng=g_strlen(arg_opts[l].name);
 						if (g_strncasecmp(argv[i]+2,arg_opts[l].name,leng)==0
 											&& ((argv[i])[leng+2]=='\0'
 												|| (argv[i])[leng+2]=='=')) {
@@ -75,7 +72,7 @@ gboolean *arg_analyse(gchar **argv,const gboolean boot)
 						}
 					}
 				} else {					/* ûץ */
-					opt=tolower((argv[i])[j++]);
+					opt=g_ascii_tolower((argv[i])[j++]);
 				}
 				arg=NULL;
 				for (l=0;arg_opts[l].name!=NULL;l++)
@@ -120,7 +117,7 @@ gboolean *arg_analyse(gchar **argv,const gboolean boot)
 						break;
 					case 's':
 						text=g_strdup(arg);
-						for (l=0;isdigit(text[l]);l++);
+						for (l=0;g_ascii_isdigit(text[l]);l++);
 						if (text[l]!='\0') {
 							text[l]='\0';
 							if (misc_str_to_val(&value,text,10,TRUE)
@@ -141,19 +138,19 @@ gboolean *arg_analyse(gchar **argv,const gboolean boot)
 					case 'v':
 						if (boot) {
 							g_print(
-"Video maid "VERSION"\n"
-"Copyright (C) 1998-2002 Kazuki IWAMOTO\n"
+VMAID_VERSION_NOTICE"\n"
+VMAID_COPYRIGHT_NOTICE"\n"
 "This is free software; see the source for copying conditions.  There is NO\n"
 "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
 							);
 							g_free(arg_files);
-							return 0;
+							return NULL;
 						}
 						break;
 					default:
 						if (boot) {
 							g_print(
-"Copyright (c) 1998-2002 Kazuki IWAMOTO\n\n"
+VMAID_COPYRIGHT_NOTICE"\n\n"
 "Usage: %s [OPTION...] [FILE...]\n\n",argv[0]);
 							for (i=0;arg_opts[i].name;i++) {
 								if (arg_opts[i].arg!=NULL)
@@ -169,7 +166,7 @@ gboolean *arg_analyse(gchar **argv,const gboolean boot)
 							}
 							g_print("\n");
 							g_free(arg_files);
-							return 0;
+							return NULL;
 						}
 				}
 			}
