@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2002 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2003 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,9 +16,8 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#include <pthread.h>
-#include "command.h"
 #include "general.h"
+#include "command.h"
 #include "signal.h"
 #include "thread.h"
 #include "toolbar.h"
@@ -46,7 +45,7 @@ void hide_caret(VmaidWindow *vmaid)
 									&& vmaid->cursor.frame<=vmaid->top+sx) {
 			rc.x=(vmaid->cursor.frame-vmaid->top+1)*vmaid->width+1;
 			rc.y=system_font_height*2
-								+(vmaid->avi[0]!=NULL && vmaid->cursor.stream>0
+						+(vmaid->avi_edit[0]!=NULL && vmaid->cursor.stream>0
 															?vmaid->height:0);
 			rc.width=2;
 			rc.height=vmaid->height;
@@ -76,7 +75,8 @@ void draw_caret(VmaidWindow *vmaid,VmaidCursor *cursor)
 		if (vmaid->top<=cursor->frame && cursor->frame<=vmaid->top+sx) {
 			rc.x=(cursor->frame-vmaid->top+1)*vmaid->width+1;
 			rc.y=system_font_height*2
-					+(vmaid->avi[0]!=NULL && cursor->stream>0?vmaid->height:0);
+								+(vmaid->avi_edit[0]!=NULL && cursor->stream>0
+															?vmaid->height:0);
 			rc.width=2;
 			rc.height=vmaid->height;
 			gtk_widget_draw(vmaid->drawing,&rc);
@@ -89,7 +89,8 @@ void draw_caret(VmaidWindow *vmaid,VmaidCursor *cursor)
 									&& vmaid->cursor.frame<=vmaid->top+sx) {
 		rc.x=(vmaid->cursor.frame-vmaid->top+1)*vmaid->width+1;
 		rc.y=system_font_height*2
-			+(vmaid->avi[0]!=NULL && vmaid->cursor.stream>0?vmaid->height:0);
+						+(vmaid->avi_edit[0]!=NULL && vmaid->cursor.stream>0
+															?vmaid->height:0);
 		rc.width=2;
 		rc.height=vmaid->height;
 		gtk_widget_draw(vmaid->drawing,&rc);
@@ -129,13 +130,13 @@ void set_menu_bar(VmaidWindow *vmaid)
 				"<main>/File/Close"),TRUE);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 				"<main>/File/Save As..."),
-								vmaid->avi[0]!=NULL || vmaid->avi[1]!=NULL);
+						vmaid->avi_edit[0]!=NULL || vmaid->avi_edit[1]!=NULL);
 //		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 //				"<main>/File/Preview..."),
-//								vmaid->avi[0]!=NULL || vmaid->avi[1]!=NULL);
+//						vmaid->avi_edit[0]!=NULL || vmaid->avi_edit[1]!=NULL);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 				"<main>/File/Properties..."),
-								vmaid->avi[0]!=NULL || vmaid->avi[1]!=NULL);
+						vmaid->avi_edit[0]!=NULL || vmaid->avi_edit[1]!=NULL);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 				"<main>/Edit/Undo"),vmaid->undo!=NULL);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
@@ -148,15 +149,15 @@ void set_menu_bar(VmaidWindow *vmaid)
 				"<main>/Edit/Delete"),vmaid->select.stream>=0);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 				"<main>/Edit/Jump..."),
-								vmaid->avi[0]!=NULL || vmaid->avi[1]!=NULL);
+						vmaid->avi_edit[0]!=NULL || vmaid->avi_edit[1]!=NULL);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory_menu,
 				"<main>/Edit/All"),
-								vmaid->avi[0]!=NULL || vmaid->avi[1]!=NULL);
+						vmaid->avi_edit[0]!=NULL || vmaid->avi_edit[1]!=NULL);
 		gtk_widget_set_sensitive(toolbar_items[ 2].item,TRUE);
 		gtk_widget_set_sensitive(toolbar_items[ 4].item,
-								vmaid->avi[0]!=NULL || vmaid->avi[1]!=NULL);
+						vmaid->avi_edit[0]!=NULL || vmaid->avi_edit[1]!=NULL);
 //		gtk_widget_set_sensitive(toolbar_items[ 6].item,
-//								vmaid->avi[0]!=NULL || vmaid->avi[1]!=NULL);
+//						vmaid->avi_edit[0]!=NULL || vmaid->avi_edit[1]!=NULL);
 		gtk_widget_set_sensitive(toolbar_items[ 8].item,
 								vmaid->select.stream>=0);
 		gtk_widget_set_sensitive(toolbar_items[ 9].item,
@@ -219,8 +220,8 @@ void clear_sel(VmaidWindow *vmaid,
 		rc.x=vmaid->width+2;
 		rc.y=0;
 		rc.width=vmaid->width*sx;
-		rc.height=system_font_height*2+((vmaid->avi[0]!=NULL?1:0)
-									+(vmaid->avi[1]!=NULL?1:0))*vmaid->height;
+		rc.height=system_font_height*2+((vmaid->avi_edit[0]!=NULL?1:0)
+								+(vmaid->avi_edit[1]!=NULL?1:0))*vmaid->height;
 		if (ABS(top-vmaid->top)<=sx) {
 			/* ư̤Ȥ */
 			dx=(top-vmaid->top)*vmaid->width;
@@ -241,7 +242,7 @@ void clear_sel(VmaidWindow *vmaid,
 			rc0.x=vmaid->cursor.frame==vmaid->top || vmaid->cursor.frame==top
 										?vmaid->width+1:vmaid->width*(sx+1)+2;
 			rc0.y=system_font_height*2
-								+(vmaid->avi[0]!=NULL && vmaid->cursor.stream>0
+						+(vmaid->avi_edit[0]!=NULL && vmaid->cursor.stream>0
 															?vmaid->height:0);
 			rc0.width=1;
 			rc0.height=vmaid->height;
@@ -254,7 +255,7 @@ void clear_sel(VmaidWindow *vmaid,
 			rc0.x=cursor->frame==vmaid->top || cursor->frame==top
 										?vmaid->width+1:vmaid->width*(sx+1)+2;
 			rc0.y=system_font_height*2
-								+(vmaid->avi[0]!=NULL && cursor->stream>0
+								+(vmaid->avi_edit[0]!=NULL && cursor->stream>0
 															?vmaid->height:0);
 			rc0.width=1;
 			rc0.height=vmaid->height;
@@ -284,7 +285,8 @@ void clear_sel(VmaidWindow *vmaid,
 	if (left_old!=left_new || right_old!=right_new) {
 		rc0.y=0;
 		rc0.width=vmaid->width;
-		rc0.height=((vmaid->avi[0]!=NULL?1:0)+(vmaid->avi[1]!=NULL?1:0))
+		rc0.height
+			=((vmaid->avi_edit[0]!=NULL?1:0)+(vmaid->avi_edit[1]!=NULL?1:0))
 										*vmaid->height+system_font_height*2;
 		/* ¦ˤĤ */
 		if (left_old!=left_new) {
@@ -318,7 +320,7 @@ void clear_sel(VmaidWindow *vmaid,
 														+system_font_height*2;
 			rc0.width=width*vmaid->width;
 			rc0.height=(ABS(cursor->stream-select->stream)+1)*vmaid->height;
-			if (vmaid->avi[0]==NULL)
+			if (vmaid->avi_edit[0]==NULL)
 				rc0.y-=vmaid->height;
 			gtk_widget_draw(vmaid->drawing,&rc);
 			if (rc.x<0) {
@@ -360,7 +362,7 @@ void clear_sel(VmaidWindow *vmaid,
 				rc0.width=width*vmaid->width;
 				rc0.height=(ABS(vmaid->cursor.stream-vmaid->select.stream)+1)
 																*vmaid->height;
-				if (vmaid->avi[0]==NULL)
+				if (vmaid->avi_edit[0]==NULL)
 					rc0.y-=vmaid->height;
 				if (rc.x<0) {
 					rc=rc0;
@@ -386,8 +388,8 @@ gint delete_list(VmaidHistory **d)
 
 	for (d0=*d;d0!=NULL;d0=d1) {
 		for (i=0;i<2;i++)
-			if (d0->avi[i]!=NULL)
-				avi_release(d0->avi[i]);
+			if (d0->avi_edit[i]!=NULL)
+				avi_release(d0->avi_edit[i]);
 		d1=d0->next;
 		g_free(d0);
 		count++;
@@ -415,11 +417,11 @@ void history_operation(VmaidWindow *vmaid,const gboolean kind)
 	e.cursor=vmaid->cursor;
 	e.select=vmaid->select;
 	for (i=0;i<2;i++)
-		e.avi[i]=vmaid->avi[i];
+		e.avi_edit[i]=vmaid->avi_edit[i];
 	vmaid->cursor=d->cursor;
 	vmaid->select=d->select;
 	for (i=0;i<2;i++)
-		vmaid->avi[i]=d->avi[i];
+		vmaid->avi_edit[i]=d->avi_edit[i];
 	*d=e;
 	if (kind) {
 		d->next=vmaid->undo;
@@ -468,6 +470,21 @@ gboolean prompt_close(VmaidWindow *vmaid)
 }
 
 
+/*	֤ԽѤߥޡѹ
+	  vmaid,ɥ
+	is_edit,TRUE:ԽѤ,FALSE:̤Խ										*/
+void
+vmaid_change_edit(VmaidWindow *vmaid,gboolean is_edit)
+{
+	if (vmaid->edit != is_edit)
+	  {
+		vmaid->edit = is_edit;
+		if (is_edit)
+			gtk_widget_show (vmaid->mark);
+		else
+			gtk_widget_hide (vmaid->mark);
+	  }
+}
 /******************************************************************************
 *                                                                             *
 * ٥ؿ                                                              *
@@ -481,9 +498,9 @@ gint get_max_frame(VmaidWindow *vmaid,const gint stream)
 {
 	gint max0,max1;
 
-	max0=vmaid->avi[0]!=NULL?avi_length(vmaid->avi[0])-1:0;
-	max1=vmaid->avi[1]!=NULL?(gint64)avi_length_time(vmaid->avi[1])*vmaid->rate
-											/((gint64)vmaid->scale*1000)-1:0;
+	max0=vmaid->avi_edit[0]!=NULL?avi_length(vmaid->avi_edit[0])-1:0;
+	max1=vmaid->avi_edit[1]!=NULL?(gint64)avi_length_time(vmaid->avi_edit[1])
+								*vmaid->rate/((gint64)vmaid->scale*1000)-1:0;
 	switch (stream) {
 		case 0:return max0;
 		case 1:return max1;
