@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2002 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2003 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,18 +16,15 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#include <pthread.h>
-#include <string.h>
+#include "signal.h"
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkrgb.h>
 #include "argument.h"
 #include "command.h"
 #include "file.h"
 #include "general.h"
-#include "signal.h"
 #include "thread.h"
-#include "avicore/avicore.h"
 #include "misc/misc.h"
-#include <gdk/gdkrgb.h>
 
 
 /******************************************************************************
@@ -50,7 +47,7 @@ gboolean signal_timeout(gpointer data)
 									&& vmaid->cursor.frame<=vmaid->top+sx) {
 			rc.x=(vmaid->cursor.frame-vmaid->top+1)*vmaid->width+1;
 			rc.y=system_font_height*2
-								+(vmaid->avi[0]!=NULL && vmaid->cursor.stream>0
+						+(vmaid->avi_edit[0]!=NULL && vmaid->cursor.stream>0
 															?vmaid->height:0);
 			rc.width=2;
 			rc.height=vmaid->height;
@@ -115,8 +112,9 @@ gboolean signal_config(GtkWidget *widget,GdkEventConfigure *config,
 	if (vmaid->top>max-sx)
 		vmaid->top=MAX(max-sx,0);
 	draw_caret(vmaid,NULL);
-	misc_set_scroll_bar(vmaid->hscroll,signal_value_changed_hscroll,vmaid,
-														0,max,sx,vmaid->top);
+	misc_set_scroll_bar(vmaid->hscroll,
+								GTK_SIGNAL_FUNC(signal_value_changed_hscroll),
+													vmaid,0,max,sx,vmaid->top);
 	return TRUE;
 }
 
@@ -127,8 +125,8 @@ static void signal_expose_text(GtkWidget *widget,GdkGC *gc,VmaidWindow *vmaid,
 	gchar *text;
 	gint t;
 
-	text=g_strdup_printf(vmaid->avi[0]!=NULL
-						&& avi_is_keyframe(vmaid->avi[0],pos)?"%d*":"%d",pos);
+	text=g_strdup_printf(vmaid->avi_edit[0]!=NULL
+					&& avi_is_keyframe(vmaid->avi_edit[0],pos)?"%d*":"%d",pos);
 	if (gdk_string_width(system_font,text)<=vmaid->width)
 		gdk_draw_string(widget->window,system_font,gc,
 													x,system_font_ascent,text);
@@ -142,35 +140,20 @@ static void signal_expose_text(GtkWidget *widget,GdkGC *gc,VmaidWindow *vmaid,
 }
 
 
-static void signal_expose_bitmap(GtkWidget *widget,GdkGC *gc,AviEdit *avi,
-								gint pos,gint x,gint y,gint width,gint height)
+static void signal_expose_bitmap(GtkWidget *widget,GdkGC *gc,
+			AviFrame *avi_frame,gint pos,gint x,gint y,gint width,gint height)
 {
-	guchar *buf,*p;
-	gint xx,yy,src,dst;
-	BitmapInfoHeader *bmih;
+	guchar *buf;
 
-	if ((bmih=avi_get_frame(avi,pos,width,height))!=NULL) {
-		buf=g_malloc(width*height*sizeof(guint32));
-		p=(guchar *)bmih+bm_header_bytes(bmih);
-		for (yy=0;yy<height;yy++)
-			for (xx=0;xx<width;xx++) {
-				src=xx*4+(height-yy-1)*width*4;
-				dst=xx*4+yy*width*4;
-				buf[dst]  =p[src+2];
-				buf[dst+1]=p[src+1];
-				buf[dst+2]=p[src];
-				buf[dst+3]=p[src+3];
-			}
+	if ((buf=avi_get_frame32(avi_frame,pos,width,height))!=NULL)
 		gdk_draw_rgb_32_image(widget->window,gc,x,y,width,height,
 											GDK_RGB_DITHER_NORMAL,buf,width*4);
-		g_free(buf);
-	} else {
+	else
 		gdk_draw_rectangle(widget->window,gc,TRUE,x,y,width,height);
-	}
 }
 
 
-static void signal_expose_wave(GtkWidget *widget,GdkGC *gc,AviEdit *avi,
+static void signal_expose_wave(GtkWidget *widget,GdkGC *gc,AviEdit *avi_edit,
 							WaveFormatEx *wfx,gint current_time,gint next_time,
 										gint x,gint y,gint width,gint height)
 {
@@ -179,17 +162,17 @@ static void signal_expose_wave(GtkWidget *widget,GdkGC *gc,AviEdit *avi,
 	GdkPoint pt[5];
 
 	start=0;
-	end=avi_length_time(avi);
+	end=avi_length_time(avi_edit);
 	if (wfx!=NULL) {
 		/* ϰPCMɤ߹ */
-		pos=avi_time_to_sample(avi,MAX(start,current_time));
-		samples=avi_time_to_sample(avi,MIN(end,next_time))-pos;
-		if (samples>avi_length(avi)-pos)
-			samples=avi_length(avi)-pos;
-		length=avi_sample_size(avi,pos,samples);
+		pos=avi_time_to_sample(avi_edit,MAX(start,current_time));
+		samples=avi_time_to_sample(avi_edit,MIN(end,next_time))-pos;
+		if (samples>avi_length(avi_edit)-pos)
+			samples=avi_length(avi_edit)-pos;
+		length=avi_sample_size(avi_edit,pos,samples);
 		if (length>0) {
 			p=g_malloc(length);
-			if (!avi_read(avi,pos,samples,p))
+			if (!avi_read(avi_edit,pos,samples,p))
 				length=0;
 		}
 	}
@@ -279,6 +262,7 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 															VmaidWindow *vmaid)
 {
 	gint i,sx,max,frame,size;
+	AviFrame *avi_frame;
 	WaveFormatEx *wfx;
 	GdkGC *gc;
 	GdkRectangle rc0,rc1;
@@ -310,14 +294,15 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 		signal_expose_text(widget,gc,vmaid,frame,vmaid->width*(sx+1)+4,0);
 	rc0.y=system_font_height*2;
 	rc0.height=vmaid->height;
-	if (vmaid->avi[0]!=NULL) {
+	if (vmaid->avi_edit[0]!=NULL) {
+		avi_frame=avi_get_frame_open(vmaid->avi_edit[0]);
 		max=MIN(get_max_frame(vmaid,0),vmaid->top+sx-1);
 		rc0.x=0;
 		if (frame>=0 && frame<vmaid->top) {
 			/* 򳫻ϰ֤ΤȤ */
 			rc0.width=vmaid->width;
 			if (gdk_rectangle_intersect(&event->area,&rc0,&rc1)) {
-				signal_expose_bitmap(widget,gc,vmaid->avi[0],frame,
+				signal_expose_bitmap(widget,gc,avi_frame,frame,
 											rc0.x,rc0.y,rc0.width,rc0.height);
 			}
 			rc0.x=vmaid->width;
@@ -333,7 +318,7 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 		rc0.width=vmaid->width;
 		for (i=vmaid->top,rc0.x=vmaid->width+2;i<=max;i++,rc0.x+=vmaid->width)
 			if (gdk_rectangle_intersect(&event->area,&rc0,&rc1))
-				signal_expose_bitmap(widget,gc,vmaid->avi[0],i,
+				signal_expose_bitmap(widget,gc,avi_frame,i,
 											rc0.x,rc0.y,rc0.width,rc0.height);
 		rc0.width=vmaid->width*(sx+1)+4-rc0.x;
 		if (gdk_rectangle_intersect(&event->area,&rc0,&rc1)) {
@@ -346,7 +331,7 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 		if (vmaid->top+sx<=frame
 						&& gdk_rectangle_intersect(&event->area,&rc0,&rc1)) {
 			/* 򳫻ϰ֤ΤȤ */
-			signal_expose_bitmap(widget,gc,vmaid->avi[0],frame,
+			signal_expose_bitmap(widget,gc,avi_frame,frame,
 										rc0.x,rc0.y,vmaid->width,rc0.height);
 			rc0.x+=vmaid->width;
 		}
@@ -357,8 +342,9 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 											rc1.x,rc1.y,rc1.width,rc1.height);
 		}
 		rc0.y+=vmaid->height;
+		avi_get_frame_close(avi_frame);
 	}
-	if (vmaid->avi[1]!=NULL) {
+	if (vmaid->avi_edit[1]!=NULL) {
 		/* ط */
 		rc0.x=event->area.x;
 		rc0.width=event->area.width;
@@ -367,9 +353,9 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 			gdk_draw_rectangle(widget->window,gc,TRUE,
 											rc1.x,rc1.y,rc1.width,rc1.height);
 		}
-		size=avi_format_size(vmaid->avi[1]);
+		size=avi_format_size(vmaid->avi_edit[1]);
 		wfx=g_malloc(size);
-		avi_read_format(vmaid->avi[1],wfx);
+		avi_read_format(vmaid->avi_edit[1],wfx);
 		if (wfx_get_format_tag(wfx)!=WAVE_FORMAT_PCM
 				|| (wfx_get_channels(wfx)!=1 && wfx_get_channels(wfx)!=2)
 				|| (wfx_get_bits_per_sample(wfx)!=8
@@ -385,23 +371,23 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 		if (frame>=0 && frame<vmaid->top
 							&& gdk_rectangle_intersect(&event->area,&rc0,&rc1))
 			/* 򳫻ϰ֤ΤȤ */
-			signal_expose_wave(widget,gc,vmaid->avi[1],wfx,
-							(gint64)frame*vmaid->scale*1000/vmaid->rate,
-							(gint64)(frame+1)*vmaid->scale*1000/vmaid->rate,
+			signal_expose_wave(widget,gc,vmaid->avi_edit[1],wfx,
+						(gint)(gint64)frame*vmaid->scale*1000/vmaid->rate,
+						(gint)(gint64)(frame+1)*vmaid->scale*1000/vmaid->rate,
 											rc0.x,rc0.y,rc0.width,rc0.height);
 		for (i=vmaid->top,rc0.x=vmaid->width+2;i<=max;i++,rc0.x+=vmaid->width)
 			if (gdk_rectangle_intersect(&event->area,&rc0,&rc1))
-				signal_expose_wave(widget,gc,vmaid->avi[1],wfx,
-								(gint64)i*vmaid->scale*1000/vmaid->rate,
-								(gint64)(i+1)*vmaid->scale*1000/vmaid->rate,
+				signal_expose_wave(widget,gc,vmaid->avi_edit[1],wfx,
+							(gint)(gint64)i*vmaid->scale*1000/vmaid->rate,
+							(gint)(gint64)(i+1)*vmaid->scale*1000/vmaid->rate,
 											rc0.x,rc0.y,rc0.width,rc0.height);
 		rc0.x=vmaid->width*(sx+1)+4;
 		if (vmaid->top+sx<=frame
 							&& gdk_rectangle_intersect(&event->area,&rc0,&rc1))
 			/* 򳫻ϰ֤ΤȤ */
-			signal_expose_wave(widget,gc,vmaid->avi[1],wfx,
-							(gint64)frame*vmaid->scale*1000/vmaid->rate,
-							(gint64)(frame+1)*vmaid->scale*1000/vmaid->rate,
+			signal_expose_wave(widget,gc,vmaid->avi_edit[1],wfx,
+						(gint)(gint64)frame*vmaid->scale*1000/vmaid->rate,
+						(gint)(gint64)(frame+1)*vmaid->scale*1000/vmaid->rate,
 											rc0.x,rc0.y,rc0.width,rc0.height);
 		g_free(wfx);
 	}
@@ -416,7 +402,7 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 			rc0.x=(rc0.x-vmaid->top+1)*vmaid->width+2;
 			rc0.y=MIN(vmaid->cursor.stream,vmaid->select.stream);
 			rc0.height=MAX(vmaid->cursor.stream,vmaid->select.stream)-rc0.y+1;
-			if (vmaid->avi[0]==NULL)
+			if (vmaid->avi_edit[0]==NULL)
 				rc0.y--;
 			rc0.y=rc0.y*vmaid->height+system_font_height*2;
 			rc0.height*=vmaid->height;
@@ -436,7 +422,8 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 									&& vmaid->cursor.frame<=vmaid->top+sx) {
 		rc0.x=(vmaid->cursor.frame-vmaid->top+1)*vmaid->width+1;
 		rc0.y=system_font_height*2
-			+(vmaid->avi[0]!=NULL && vmaid->cursor.stream>0?vmaid->height:0);
+						+(vmaid->avi_edit[0]!=NULL && vmaid->cursor.stream>0
+															?vmaid->height:0);
 		rc0.width=2;
 		rc0.height=vmaid->height;
 		if (gdk_rectangle_intersect(&event->area,&rc0,&rc1)) {
@@ -450,7 +437,7 @@ gboolean signal_expose(GtkWidget *widget,GdkEventExpose *event,
 }
 
 
-gboolean signal_button_press(GtkWidget *widget,GdkEventButton *event,
+gboolean signal_button_press_draw(GtkWidget *widget,GdkEventButton *event,
 															VmaidWindow *vmaid)
 {
 	if (!GTK_WIDGET_HAS_FOCUS(widget))
@@ -464,7 +451,7 @@ gboolean signal_button_press(GtkWidget *widget,GdkEventButton *event,
 				gint max,sx;
 				VmaidCursor cursor,select;
 
-				if (vmaid->avi[0]==NULL && vmaid->avi[1]==NULL)
+				if (vmaid->avi_edit[0]==NULL && vmaid->avi_edit[1]==NULL)
 					break;
 				cursor=vmaid->cursor;
 				select=vmaid->select;
@@ -480,7 +467,7 @@ gboolean signal_button_press(GtkWidget *widget,GdkEventButton *event,
 					vmaid->cursor.frame=vmaid->top+sx;
 				if (vmaid->cursor.frame>max)
 					vmaid->cursor.frame=max;
-				if (vmaid->avi[0]!=NULL && vmaid->avi[1]!=NULL)
+				if (vmaid->avi_edit[0]!=NULL && vmaid->avi_edit[1]!=NULL)
 					vmaid->cursor.stream
 							=event->y<system_font_height*2+vmaid->height?0:1;
 				if (vmaid->select.stream>=0 && (!shift
@@ -531,7 +518,7 @@ gboolean signal_button_press(GtkWidget *widget,GdkEventButton *event,
 					vmaid->top=MAX(max-sx,0);
 				if (vmaid->top!=top)
 					misc_set_scroll_bar(vmaid->hscroll,
-												signal_value_changed_hscroll,
+								GTK_SIGNAL_FUNC(signal_value_changed_hscroll),
 													vmaid,0,max,sx,vmaid->top);
 				clear_sel(vmaid,&vmaid->cursor,&vmaid->select,top);
 			}
@@ -579,7 +566,7 @@ static gboolean signal_motion_notify_draw(VmaidWindow *vmaid,gint x,gint y,
 		vmaid->cursor.frame=0;
 	else if (vmaid->cursor.frame>max)
 		vmaid->cursor.frame=max;
-	if (vmaid->avi[0]!=NULL && vmaid->avi[1]!=NULL)
+	if (vmaid->avi_edit[0]!=NULL && vmaid->avi_edit[1]!=NULL)
 		vmaid->cursor.stream=ps.y<system_font_height*2+vmaid->height?0:1;
 	if (vmaid->cursor.frame==cursor.frame
 										&& vmaid->cursor.stream==cursor.stream)
@@ -592,8 +579,9 @@ static gboolean signal_motion_notify_draw(VmaidWindow *vmaid,gint x,gint y,
 	if (vmaid->select.stream!=select.stream)
 		set_menu_bar(vmaid);
 	if (vmaid->top!=top)
-		misc_set_scroll_bar(vmaid->hscroll,signal_value_changed_hscroll,vmaid,
-														0,max,sx,vmaid->top);
+		misc_set_scroll_bar(vmaid->hscroll,
+								GTK_SIGNAL_FUNC(signal_value_changed_hscroll),
+													vmaid,0,max,sx,vmaid->top);
 	clear_sel(vmaid,&cursor,&select,top);
 	draw_caret(vmaid,&cursor);
 	return vmaid->top!=top;
@@ -673,7 +661,7 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,
 			gint sx,max,top;
 			VmaidCursor cursor,select;
 
-			if (vmaid->avi[0]==NULL || vmaid->avi[1]==NULL)
+			if (vmaid->avi_edit[0]==NULL || vmaid->avi_edit[1]==NULL)
 				break;
 			cursor=vmaid->cursor;
 			select=vmaid->select;
@@ -692,7 +680,7 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,
 			}
 			if (vmaid->top!=top)
 				misc_set_scroll_bar(vmaid->hscroll,
-												signal_value_changed_hscroll,
+								GTK_SIGNAL_FUNC(signal_value_changed_hscroll),
 													vmaid,0,max,sx,vmaid->top);
 			clear_sel(vmaid,&cursor,&select,top);
 			draw_caret(vmaid,&cursor);
@@ -718,7 +706,7 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,
 			gint sx,max,top;
 			VmaidCursor cursor,select;
 
-			if (vmaid->avi[0]==NULL && vmaid->avi[1]==NULL)
+			if (vmaid->avi_edit[0]==NULL && vmaid->avi_edit[1]==NULL)
 				break;
 			cursor=vmaid->cursor;
 			select=vmaid->select;
@@ -802,7 +790,7 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,
 				set_menu_bar(vmaid);
 			if (vmaid->top!=top)
 				misc_set_scroll_bar(vmaid->hscroll,
-												signal_value_changed_hscroll,
+								GTK_SIGNAL_FUNC(signal_value_changed_hscroll),
 													vmaid,0,max,sx,vmaid->top);
 			clear_sel(vmaid,&cursor,&select,top);
 			draw_caret(vmaid,&cursor);
@@ -831,7 +819,7 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,
 			set_menu_bar(vmaid);
 			if (vmaid->top!=top)
 				misc_set_scroll_bar(vmaid->hscroll,
-												signal_value_changed_hscroll,
+								GTK_SIGNAL_FUNC(signal_value_changed_hscroll),
 							vmaid,0,get_max_frame(vmaid,-1)+1,sx,vmaid->top);
 			clear_sel(vmaid,&cursor,&select,top);
 			draw_caret(vmaid,&cursor);
@@ -846,7 +834,7 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,
 			gboolean ctrl;
 			gint max;
 
-			if (vmaid->avi[0]==NULL && vmaid->avi[1]==NULL)
+			if (vmaid->avi_edit[0]==NULL && vmaid->avi_edit[1]==NULL)
 				break;
 			ctrl=(event->state&GDK_CONTROL_MASK)!=0;
 			if (vmaid->select.stream<0) {
@@ -880,7 +868,7 @@ gboolean signal_key_press(GtkWidget *widget,GdkEventKey *event,
 				vmaid->top=MAX(vmaid->cursor.frame-sx,0);
 			if (vmaid->top!=top)
 				misc_set_scroll_bar(vmaid->hscroll,
-												signal_value_changed_hscroll,
+								GTK_SIGNAL_FUNC(signal_value_changed_hscroll),
 							vmaid,0,get_max_frame(vmaid,-1)+1,sx,vmaid->top);
 			clear_sel(vmaid,&vmaid->cursor,&vmaid->select,top);
 			gtk_widget_set_sensitive(gtk_item_factory_get_widget(
@@ -954,8 +942,8 @@ void signal_destroy_draw(GtkWidget *widget,VmaidWindow *vmaid)
 		vmaid->timer_id=0;
 	}
 	for (i=0;i<2;i++)
-		if (vmaid->avi[i]!=NULL)
-			avi_release(vmaid->avi[i]);
+		if (vmaid->avi_edit[i]!=NULL)
+			avi_release(vmaid->avi_edit[i]);
 	delete_list(&vmaid->undo);
 	delete_list(&vmaid->redo);
 	g_free(vmaid);
@@ -982,13 +970,13 @@ void signal_style_set(GtkWidget *widget,GtkStyle *style,gpointer user_data)
 	if (system_font!=NULL)
 		gdk_font_unref(system_font);
 	/* ƥե */
-	system_font=gdk_font_ref(style->font);
+	system_font=gdk_font_ref(gtk_style_get_font(style));
 	for (i=0;i<256;i++) {
 		gdk_text_extents(system_font,(gchar *)&i,1,
 												NULL,NULL,NULL,&ascent,NULL);
 		if (system_font_ascent<ascent)
 			system_font_ascent=ascent;
-		height=gdk_char_height(system_font,i);
+		height=gdk_char_height(system_font,(gchar)i);
 		if (system_font_height<height)
 			system_font_height=height;
 	}
@@ -996,9 +984,9 @@ void signal_style_set(GtkWidget *widget,GtkStyle *style,gpointer user_data)
 	for (i=g_list_length(glist)-1;i>=0;i--) {
 		vmaid=gtk_object_get_user_data(GTK_OBJECT(g_list_nth_data(glist,i)));
 		gtk_widget_set_usize(vmaid->drawing,vmaid->width*3+4,
-								system_font_height*2+((vmaid->avi[0]!=NULL?1:0)
-									+(vmaid->avi[1]!=NULL?1:0))*vmaid->height);
-		gtk_container_queue_resize(GTK_CONTAINER(window));
+						system_font_height*2+((vmaid->avi_edit[0]!=NULL?1:0)
+							+(vmaid->avi_edit[1]!=NULL?1:0))*vmaid->height);
+		gtk_widget_queue_resize (GTK_WIDGET (window));
 		gtk_widget_draw(vmaid->drawing,NULL);
 	}
 	g_list_free(glist);
@@ -1021,14 +1009,20 @@ void signal_activate_menu_window(GtkWidget *widget,GtkWidget *child)
 
 void signal_activate_menu_history(GtkWidget *widget,gchar *file)
 {
+#ifndef USE_GTK2
 	pthread_t id;
+#endif
 	FileOpen *file_open;
 
 	file_open=g_malloc(sizeof(FileOpen));
 	file_open->file=g_strdup(file);
 	file_open->merge=file_merge;
+#ifdef USE_GTK2
+	g_thread_create((GThreadFunc)file_open_edit,file_open,TRUE,NULL);
+#else
 	pthread_create(&id,NULL,(void *)file_open_edit,file_open);
 	pthread_detach(id);
+#endif
 }
 
 
@@ -1057,7 +1051,8 @@ void signal_switch_page(GtkNotebook *notebook,GtkNotebookPage *page,
 {
 	VmaidWindow *vmaid;
 
-	vmaid=gtk_object_get_user_data(GTK_OBJECT(page->child));
+	vmaid=gtk_object_get_user_data(
+					GTK_OBJECT(gtk_notebook_get_nth_page(notebook,page_num)));
 	/* ˥塼ɽ */
 	set_menu_bar(vmaid);
 }
@@ -1074,6 +1069,29 @@ void signal_destroy_notebook(GtkWidget *widget,gpointer user_data)
 * ʥ/٥ȴؿ(ꥹ)                                             *
 *                                                                             *
 ******************************************************************************/
+#ifdef USE_GTK2
+static void
+signal_tree_selection_foreach (GtkTreeModel *model, GtkTreePath *path,
+										GtkTreeIter *iter, gpointer user_data)
+{
+	(* (gint *)user_data) ++;
+}
+
+
+void
+signal_changed (GtkTreeSelection *select, gpointer user_data)
+{
+	gint count;
+
+	gtk_tree_selection_selected_foreach (select, signal_tree_selection_foreach,
+																	&count);
+	gtk_widget_set_sensitive (gtk_menu_get_attach_widget (GTK_MENU (
+			gtk_item_factory_get_widget (item_factory_menu, "<main>/Task"))),
+																	count > 0);
+}
+
+
+#else /* not USE_GTK2 */
 void signal_select_row(GtkWidget *widget,gint row,gint column,
 									GdkEventButton *event,gpointer user_data)
 {
@@ -1099,6 +1117,27 @@ void signal_unselect_row(GtkWidget *widget,gint row,gint column,
 }
 
 
+gboolean signal_button_press_clist(GtkWidget *widget,GdkEventButton *event,
+															gpointer user_data)
+{
+	gint row,column;
+
+	if (event->type==GDK_BUTTON_PRESS && event->button==3
+			&& gtk_clist_get_selection_info(GTK_CLIST(clist),
+								(gint)event->x,(gint)event->y,&row,&column)) {
+		/* å */
+		gtk_clist_unselect_all(GTK_CLIST(clist));
+		gtk_clist_select_row(GTK_CLIST(clist),row,column);
+		gtk_menu_popup(GTK_MENU(
+					gtk_item_factory_get_widget(item_factory_clist,"<main>")),
+								NULL,NULL,NULL,NULL,event->button,event->time);
+		return TRUE;
+	}
+	return FALSE;
+}
+#endif /* not USE_GTK2 */
+
+
 /******************************************************************************
 *                                                                             *
 * ʥ/٥ȴؿ(쥯)                                       *
@@ -1109,17 +1148,23 @@ void signal_drag_data_received(GtkWidget *widget,GdkDragContext *context,
 {
 	gchar **files;
 	gint i;
+#ifndef USE_GTK2
 	pthread_t id;
+#endif
 	FileOpen *file_open;
 
 	files=g_strsplit(selection_data->data,"\r\n",G_MAXINT);
 	for (i=0;files[i]!=NULL;i++)
-		if (strncmp(files[i],"file:",5)==0) {
+		if (g_strncmp(files[i],"file:",5)==0) {
 			file_open=g_malloc(sizeof(FileOpen));
 			file_open->file=g_strdup(files[i]+5);
 			file_open->merge=file_merge;
+#ifdef USE_GTK2
+			g_thread_create((GThreadFunc)file_open_edit,file_open,TRUE,NULL);
+#else
 			pthread_create(&id,NULL,(void *)file_open_edit,file_open);
 			pthread_detach(id);
+#endif
 		}
 	g_strfreev(files);
 }
@@ -1146,18 +1191,18 @@ void signal_selection_get(GtkWidget *widget,GtkSelectionData *data,
 		/* åץܡ */
 		if (clipboard_scenario!=NULL && info==TARGET_VMAID)
 			gtk_selection_data_set(data,atom_scenario,8,
-					(guchar *)clipboard_scenario,strlen(clipboard_scenario));
+					(guchar *)clipboard_scenario,g_strlen(clipboard_scenario));
 	} else if (data->selection==atom_videomaid0) {
 		/* ץ̿ */
 		text="Video maid Selection";
 		gtk_selection_data_set(data,GDK_SELECTION_TYPE_STRING,8,
-												(guchar *)text,strlen(text));
+												(guchar *)text,g_strlen(text));
 		gtk_selection_convert(window,atom_videomaid1,
 								GDK_SELECTION_TYPE_STRING,GDK_CURRENT_TIME);
 	} else if (data->selection==atom_videomaid1 && arg_text!=NULL) {
 		/* ץ̿ */
 		gtk_selection_data_set(data,GDK_SELECTION_TYPE_STRING,8,
-										(guchar *)arg_text,strlen(arg_text));
+										(guchar *)arg_text,g_strlen(arg_text));
 		g_free(arg_text);
 		arg_text=NULL;
 	}
@@ -1170,8 +1215,10 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 	gboolean result=TRUE,paste[2],*arg_files;
 	gchar **argv;
 	gint i,count,max,sx;
+#ifndef USE_GTK2
 	pthread_t id;
-	AviEdit **avi;
+#endif
+	AviEdit **avi_edit;
 	FileOpen *file_open;
 	GdkAtom *atoms;
 	VmaidWindow *vmaid;
@@ -1191,7 +1238,7 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 															GDK_CURRENT_TIME);
 			} else if (data->type==atom_scenario) {
 				/* Žդ */
-				if ((avi=avi_from_scenario(data->data))==NULL)
+				if ((avi_edit=avi_from_scenario(data->data))==NULL)
 					return;
 				vmaid=gtk_object_get_user_data(GTK_OBJECT(
 					gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
@@ -1200,21 +1247,22 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 					/* ϰϤʤȤUNDOν */
 					d=g_malloc(sizeof(VmaidHistory));
 					for (i=0;i<2;i++) {
-						d->avi[i]=NULL;
-						if (vmaid->avi[i]!=NULL
-								&& (d->avi[i]=avi_clone(vmaid->avi[i]))==NULL)
+						d->avi_edit[i]=NULL;
+						if (vmaid->avi_edit[i]!=NULL
+								&& (d->avi_edit[i]
+										=avi_clone(vmaid->avi_edit[i]))==NULL)
 							result=FALSE;
 					}
 					if (!result) {
 						misc_message_box("Video maid",_("AVI Clone Error"),0,
 																_("OK"),NULL);
 						for (i=0;i<2;i++)
-							if (d->avi[i]!=NULL)
-								avi_release(d->avi[i]);
+							if (d->avi_edit[i]!=NULL)
+								avi_release(d->avi_edit[i]);
 						g_free(d);
-						for (i=0;avi[i]!=NULL;i++)
-							avi_release(avi[i]);
-						g_free(avi);
+						for (i=0;avi_edit[i]!=NULL;i++)
+							avi_release(avi_edit[i]);
+						g_free(avi_edit);
 						return;
 					}
 					d->cursor=vmaid->cursor;
@@ -1227,19 +1275,19 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 					vmaid->redo=d;
 				}
 				paste[0]=paste[1]=FALSE;
-				for (i=0;avi[i]!=NULL;i++)
-					switch (avi_type(avi[i])) {
+				for (i=0;avi_edit[i]!=NULL;i++)
+					switch (avi_type(avi_edit[i])) {
 						case streamtypeVIDEO:
 							if (paste[0]) {
-								avi_release(avi[i]);
+								avi_release(avi_edit[i]);
 							} else {
-								if (vmaid->avi[0]==NULL) {
-									vmaid->avi[0]=avi[i];
-								} else if (!avi_paste(vmaid->avi[0],
-												MIN(vmaid->cursor.frame,
-													avi_length(vmaid->avi[0])),
-																	avi[i])) {
-									avi_release(avi[i]);
+								if (vmaid->avi_edit[0]==NULL) {
+									vmaid->avi_edit[0]=avi_edit[i];
+								} else if (!avi_paste(vmaid->avi_edit[0],
+										MIN(vmaid->cursor.frame,
+											avi_length(vmaid->avi_edit[0])),
+																avi_edit[i])) {
+									avi_release(avi_edit[i]);
 									result=FALSE;
 								}
 								paste[0]=TRUE;
@@ -1247,48 +1295,49 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 							break;
 						case streamtypeAUDIO:
 							if (paste[1]) {
-								avi_release(avi[i]);
+								avi_release(avi_edit[i]);
 							} else {
-								if (vmaid->avi[1]==NULL) {
-									vmaid->avi[1]=avi[i];
-								} else if (!avi_paste(vmaid->avi[1],
-										MIN(avi_time_to_sample(vmaid->avi[1],
+								if (vmaid->avi_edit[1]==NULL) {
+									vmaid->avi_edit[1]=avi_edit[i];
+								} else if (!avi_paste(vmaid->avi_edit[1],
+									MIN(avi_time_to_sample(vmaid->avi_edit[1],
 											(gint64)vmaid->cursor.frame*1000
 													*vmaid->scale/vmaid->rate),
-										avi_length(vmaid->avi[0])),avi[i])) {
-									avi_release(avi[i]);
+										avi_length(vmaid->avi_edit[0])),
+																avi_edit[i])) {
+									avi_release(avi_edit[i]);
 									result=FALSE;
 								}
 								paste[1]=TRUE;
 							}
 							break;
 						default:
-							avi_release(avi[i]);
+							avi_release(avi_edit[i]);
 							result=FALSE;
 					}
-				g_free(avi);
+				g_free(avi_edit);
 				if (!result || (!paste[0] && !paste[1])) {
 					misc_message_box("Video maid",_("AVI Paste Error"),0,
 																_("OK"),NULL);
 					for (i=0;i<2;i++)
-						if (vmaid->avi[i]!=NULL)
-							avi_release(vmaid->avi[i]);
+						if (vmaid->avi_edit[i]!=NULL)
+							avi_release(vmaid->avi_edit[i]);
 					d=vmaid->undo;
 					vmaid->undo=vmaid->undo->next;
 					for (i=0;i<2;i++)
-						vmaid->avi[i]=d->avi[i];
+						vmaid->avi_edit[i]=d->avi_edit[i];
 					g_free(d);
 					return;
 				}
-				if (vmaid->avi[0]!=NULL) {
-					vmaid->rate=avi_rate(vmaid->avi[0]);
-					vmaid->scale=avi_scale(vmaid->avi[0]);
+				if (vmaid->avi_edit[0]!=NULL) {
+					vmaid->rate=avi_get_rate(vmaid->avi_edit[0]);
+					vmaid->scale=avi_get_scale(vmaid->avi_edit[0]);
 				}
 				max=get_max_frame(vmaid,-1)+1;
 				sx=MAX((vmaid->drawing->allocation.width-4)/vmaid->width-2,1);
-				if (vmaid->avi[0]!=NULL && vmaid->avi[1]==NULL)
+				if (vmaid->avi_edit[0]!=NULL && vmaid->avi_edit[1]==NULL)
 					vmaid->cursor.stream=0;
-				else if (vmaid->avi[0]==NULL && vmaid->avi[1]!=NULL)
+				else if (vmaid->avi_edit[0]==NULL && vmaid->avi_edit[1]!=NULL)
 					vmaid->cursor.stream=1;
 				if (vmaid->cursor.frame>max)
 					vmaid->cursor.frame=max;
@@ -1299,15 +1348,15 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 				/* ˥塼,ơȥ,ҥɥ */
 				set_menu_bar(vmaid);
 				gtk_widget_set_usize(vmaid->drawing,vmaid->width*3+4,
-								system_font_height*2+((vmaid->avi[0]!=NULL?1:0)
-									+(vmaid->avi[1]!=NULL?1:0))*vmaid->height);
-				gtk_container_queue_resize(GTK_CONTAINER(window));
+						system_font_height*2+((vmaid->avi_edit[0]!=NULL?1:0)
+							+(vmaid->avi_edit[1]!=NULL?1:0))*vmaid->height);
+				gtk_widget_queue_resize (GTK_WIDGET (window));
 				gtk_widget_draw(vmaid->drawing,NULL);
 				draw_caret(vmaid,NULL);
 				misc_set_scroll_bar(vmaid->hscroll,
-												signal_value_changed_hscroll,
+								GTK_SIGNAL_FUNC(signal_value_changed_hscroll),
 													vmaid,0,max,sx,vmaid->top);
-				vmaid->edit=TRUE;
+				vmaid_change_edit (vmaid, TRUE);
 			}
 		}
 	} else if (data->selection==atom_videomaid0) {
@@ -1318,13 +1367,18 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 		/* ޥɥ饤β */
 		argv=g_strsplit(data->data,"\n",G_MAXINT);
 		arg_files=arg_analyse(argv,FALSE);
-		for (i=0;argv[i]!=NULL;i++)
+		for (i = 1; argv[i] != NULL; i++)
 			if (arg_files[i]) {
 				file_open=g_malloc(sizeof(FileOpen));
 				file_open->file=g_strdup(argv[i]);
 				file_open->merge=file_merge;
+#ifdef USE_GTK2
+				g_thread_create((GThreadFunc)file_open_edit,
+														file_open,TRUE,NULL);
+#else
 				pthread_create(&id,NULL,(void *)file_open_edit,file_open);
 				pthread_detach(id);
+#endif
 			}
 		g_free(arg_files);
 		g_strfreev(argv);
@@ -1340,7 +1394,9 @@ void signal_selection_received(GtkWidget *widget,GtkSelectionData *data,
 gboolean signal_delete(GtkWidget *widget,GdkEvent *event,gpointer user_data)
 {
 	gint page;
+#ifndef USE_GTK2
 	pthread_t id;
+#endif
 	VmaidWindow *vmaid;
 
 	while ((page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))>=0) {
@@ -1350,8 +1406,12 @@ gboolean signal_delete(GtkWidget *widget,GdkEvent *event,gpointer user_data)
 			return TRUE;
 		gtk_notebook_remove_page(GTK_NOTEBOOK(notebook),page);
 	}
+#ifdef USE_GTK2
+	g_thread_create((GThreadFunc)close_program,NULL,TRUE,NULL);
+#else
 	pthread_create(&id,NULL,(void *)close_program,NULL);
 	pthread_detach(id);
+#endif
 	return TRUE;
 }
 
@@ -1362,7 +1422,7 @@ void signal_destroy_window(GtkWidget *widget,gpointer user_data)
 	GList *glist;
 	GtkWidget *sub_menu;
 
-	memset(history_files,0,sizeof(gchar *)*10);
+	g_memset(history_files,0,sizeof(gchar *)*10);
 	sub_menu=gtk_item_factory_get_widget(item_factory_menu,"<main>/File");
 	glist=gtk_container_children(GTK_CONTAINER(sub_menu));
 	for (i=0;i<(gint)g_list_length(glist)-MENUFILE-1;i++)
