@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2002 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2003 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,23 +16,23 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#include <ctype.h>
-#include <locale.h>
-#include <pthread.h>
-#include <string.h>
-#include <unistd.h>
 #include "vmaid.h"
+#include <gdk/gdkrgb.h>
+#include <locale.h>
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 #include "argument.h"
 #include "command.h"
 #include "conf.h"
 #include "file.h"
 #include "general.h"
 #include "signal.h"
+#include "thread.h"
 #include "toolbar.h"
 #include "version.h"
 #include "misc/misc.h"
 #include "misc/profile.h"
-#include <gdk/gdkrgb.h>
 
 
 gboolean caret=FALSE;				/* åȤη */
@@ -65,9 +65,13 @@ GtkWidget *status;					/* ơɥ */
 GtkPositionType n_pos=GTK_POS_TOP;	/* Ρȥ֥åΰ */
 GtkItemFactory *item_factory_menu;	/* ˥塼 */
 GtkItemFactory *item_factory_popup;	/* ݥåץå */
+GtkItemFactory *item_factory_clist;	/* ݥåץå */
 GdkFont *system_font=NULL;
 gint system_font_height;			/* եȤι⤵ */
-gint system_font_ascent;				/* եȤΥȤι⤵ */
+gint system_font_ascent;			/* եȤΥȤι⤵ */
+#ifdef USE_GTK2
+GtkListStore *list_store;			/* ꥹ */
+#endif /* USE_GTK2 */
 GdkColor system_color[4]={	{0,0x0000,0x0000,0x0000},{0,0xffff,0xffff,0xffff},
 							{0,0xffff,0xffff,0xffff},{0,0x0000,0x0000,0x8000}};
 GtkTargetEntry targets_vmaid[]={	/* Video maid󶡲ǽʥ쥯 */
@@ -81,9 +85,10 @@ GtkTargetEntry targets_comm[]={		/* Video maid󶡲ǽʥ쥯 */
 * ᥤؿ                                                                  *
 *                                                                             *
 ******************************************************************************/
-static gchar *menu_translate(const gchar *path,gpointer func_data)
+static gchar *
+menu_translate(const gchar *path,gpointer func_data)
 {
-   return gettext(path);
+   return (gchar *)gettext (path);
 }
 
 
@@ -92,7 +97,6 @@ int main(int argc,char *argv[])
 	gboolean *arg_files;
 	gchar *arg,*key;
 	gint i,count=0,ascent,height,leng,length=0;
-	pthread_t id;
 	FileOpen *file_open;
 	Profile *profile;
 	GdkBitmap *mask;
@@ -101,6 +105,13 @@ int main(int argc,char *argv[])
 	GtkStyle *style;
 	GtkWidget *menubar,*toolbar,*text,*handle,*scroll,*vbox;
 	GtkWidget *sub_menu,*menu_item;
+#ifdef USE_GTK2
+	GtkTreeSelection *select;
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
+#else /* not USE_GTK2 */
+	pthread_t id;
+#endif /* not USE_GTK2 */
 	gchar *title[4]={N_("Mode"),N_("File"),N_("Time"),N_("Status")};
 	GtkItemFactoryEntry menu_items[]={
 {N_("/_File"),                   NULL,         NULL,            0,"<Branch>"},
@@ -144,6 +155,8 @@ int main(int argc,char *argv[])
 {N_("/De_lete"),                 NULL,         NULL,            0,"<Item>"},
 {"/Sep2",                        NULL,         NULL,            0,"<Separator>"},
 {N_("/_All"),                    "<control>A", NULL,            0,"<Item>"}};
+	GtkItemFactoryEntry clist_items[]={
+{N_("/_Cancel"),                 NULL,         command_cancel,  0,"<Item>"}};
 	/* Drag&Drop */
 	GtkTargetEntry drag_types[1]={{"text/uri-list",0,0}};
 	static gchar *xpm0[]={
@@ -169,17 +182,25 @@ int main(int argc,char *argv[])
 	/*  */
 	setlocale(LC_ALL,"");
 	bindtextdomain(PACKAGE,LOCALEDIR);
+#ifdef USE_GTK2
+	bind_textdomain_codeset(PACKAGE,"UTF-8");
+#endif
 	textdomain(PACKAGE);
 	g_thread_init(NULL);
+#ifdef USE_GTK2
+	gdk_threads_init();
+#endif
 	gtk_set_locale();
 	gtk_init(&argc,&argv);
 	gdk_rgb_init();
+	avi_init();
+	g_atexit((GVoidFunc)avi_exit);
 
 	def_width=gdk_screen_width()*2/3;
 	def_height=gdk_screen_height()*2/3;
-	memset(history_files,0,sizeof(gchar *)*10);
+	g_memset(history_files,0,sizeof(gchar *)*10);
 	/* 쥸ȥ꤫ɤ߽Ф */
-	if ((profile=profile_open(".vmaid"))!=NULL) {
+	if ((profile=profile_open(NULL))!=NULL) {
 		if (!profile_get_value(profile,"vmaid","history",
 				(gpointer)&history,sizeof(gint),PROFILE_VALUE_TYPE_INT))
 			history=4;
@@ -231,7 +252,8 @@ int main(int argc,char *argv[])
 	}
 
 	/* ޥɥ饤β */
-	arg_files=arg_analyse(argv+1,TRUE);
+	if ((arg_files=arg_analyse(argv,TRUE))==NULL)
+		gtk_exit(0);
 
 	/* ᥤ󥦥ɥ */
 	window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -240,7 +262,7 @@ int main(int argc,char *argv[])
 	gtk_signal_connect(GTK_OBJECT(window),"delete-event",
 										GTK_SIGNAL_FUNC(signal_delete),NULL);
 	gtk_signal_connect(GTK_OBJECT(window),"destroy",
-												signal_destroy_window,NULL);
+								GTK_SIGNAL_FUNC(signal_destroy_window),NULL);
 	gtk_widget_realize(window);
 	/* Drag and Drop */
 	gtk_drag_dest_set(window,GTK_DEST_DEFAULT_MOTION
@@ -265,9 +287,9 @@ int main(int argc,char *argv[])
 	gtk_signal_connect(GTK_OBJECT(window),"selection-clear-event",
 								GTK_SIGNAL_FUNC(signal_selection_clear),NULL);
 	gtk_signal_connect(GTK_OBJECT(window),"selection-get",
-											signal_selection_get,NULL);
+								GTK_SIGNAL_FUNC(signal_selection_get),NULL);
 	gtk_signal_connect(GTK_OBJECT(window),"selection-received",
-											signal_selection_received,NULL);
+							GTK_SIGNAL_FUNC(signal_selection_received),NULL);
 	/* Ρȥ֥å */
 	notebook=gtk_notebook_new();
 	gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook),TRUE);
@@ -284,6 +306,14 @@ int main(int argc,char *argv[])
 																	NULL,NULL);
 	gtk_item_factory_create_items(item_factory_popup,
 			sizeof(popup_items)/sizeof(GtkItemFactoryEntry),popup_items,NULL);
+	/* ݥåץå */
+	accel_group=gtk_accel_group_new();
+	item_factory_clist=gtk_item_factory_new(GTK_TYPE_MENU,
+														"<main>",accel_group);
+	gtk_item_factory_set_translate_func(item_factory_clist,menu_translate,
+																	NULL,NULL);
+	gtk_item_factory_create_items(item_factory_clist,
+			sizeof(clist_items)/sizeof(GtkItemFactoryEntry),clist_items,NULL);
 	/* ˥塼 */
 	accel_group=gtk_accel_group_new();
 	item_factory_menu=gtk_item_factory_new(GTK_TYPE_MENU_BAR,
@@ -292,22 +322,54 @@ int main(int argc,char *argv[])
 																	NULL,NULL);
 	gtk_item_factory_create_items(item_factory_menu,
 			sizeof(menu_items)/sizeof(GtkItemFactoryEntry),menu_items,NULL);
-	gtk_accel_group_attach(accel_group,GTK_OBJECT(window));
+	gtk_window_add_accel_group(GTK_WINDOW(window),accel_group);
 	menubar=gtk_item_factory_get_widget(item_factory_menu,"<main>");
 	/* ġС */
+#ifdef USE_GTK2
+	toolbar=gtk_toolbar_new();
+	gtk_toolbar_set_orientation(GTK_TOOLBAR(toolbar),
+												GTK_ORIENTATION_HORIZONTAL);
+	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar),GTK_TOOLBAR_ICONS);
+#else
 	toolbar=gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,GTK_TOOLBAR_ICONS);
+#endif
 	for (i=0;toolbar_items[i].name!=(gpointer)-1;i++)
 		if (toolbar_items[i].name==NULL) {
 			gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 		} else {
 			pixmap=gdk_pixmap_create_from_xpm_d(window->window,
 											&mask,NULL,toolbar_items[i].xpm);
-			toolbar_items[i].item=gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
-						NULL,_(toolbar_items[i].name),NULL,
-						gtk_pixmap_new(pixmap,mask),
-						toolbar_items[i].callback,toolbar_items[i].user_data);
+			toolbar_items[i].item = gtk_toolbar_append_item
+						(GTK_TOOLBAR(toolbar),
+						NULL, _(toolbar_items[i].name), NULL,
+						gtk_pixmap_new (pixmap,mask),
+						GTK_SIGNAL_FUNC (toolbar_items[i].callback),
+						toolbar_items[i].user_data);
 		}
 	/* ꥹȥܥå */
+#ifdef USE_GTK2
+	list_store = gtk_list_store_new (5,
+									G_TYPE_STRING,
+									G_TYPE_STRING,
+									G_TYPE_STRING,
+									G_TYPE_STRING,
+									G_TYPE_POINTER);
+	clist = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
+	for (i = 0; i < 4; i++)
+	  {
+		renderer = gtk_cell_renderer_text_new ();
+		column = gtk_tree_view_column_new_with_attributes (gettext (title[i]),
+													renderer, "text", i, NULL);
+		gtk_tree_view_column_set_resizable (column, TRUE);
+		gtk_tree_view_append_column (GTK_TREE_VIEW (clist), column);
+	  }
+	g_object_set (G_OBJECT (renderer), "xalign", (gfloat)1.0, NULL);
+	gtk_tree_view_column_set_alignment (column, 1.0);
+	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (clist));
+	gtk_tree_selection_set_mode (select, GTK_SELECTION_MULTIPLE);
+	g_signal_connect (G_OBJECT (select), "changed",
+											G_CALLBACK (signal_changed), NULL);
+#else /* not USE_GTK2 */
 	clist=gtk_clist_new(4);
 	for (i=0;i<4;i++)
 		gtk_clist_set_column_title(GTK_CLIST(clist),i,gettext(title[i]));
@@ -315,9 +377,14 @@ int main(int argc,char *argv[])
 	gtk_clist_column_titles_passive(GTK_CLIST(clist));
 	gtk_clist_set_column_justification(GTK_CLIST(clist),3,GTK_JUSTIFY_RIGHT);
 	gtk_clist_set_selection_mode(GTK_CLIST(clist),GTK_SELECTION_MULTIPLE);
-	gtk_signal_connect(GTK_OBJECT(clist),"select-row",signal_select_row,NULL);
+	gtk_signal_connect(GTK_OBJECT(clist),"select-row",
+									GTK_SIGNAL_FUNC(signal_select_row),NULL);
 	gtk_signal_connect(GTK_OBJECT(clist),"unselect-row",
-													signal_unselect_row,NULL);
+									GTK_SIGNAL_FUNC(signal_unselect_row),NULL);
+	gtk_signal_connect(GTK_OBJECT(clist),"button-press-event",
+							GTK_SIGNAL_FUNC(signal_button_press_clist),NULL);
+	gtk_widget_add_events(clist,GDK_BUTTON_PRESS_MASK);
+#endif /* not USE_GTK2 */
 	/* 륦ɥ */
 	scroll=gtk_scrolled_window_new(NULL,NULL);
 	gtk_container_add(GTK_CONTAINER(scroll),clist);
@@ -326,8 +393,13 @@ int main(int argc,char *argv[])
 	/* ơС */
 	status=gtk_statusbar_new();
 	/* ƥ */
+#ifdef USE_GTK2
+	text=gtk_label_new("");
+#else
 	text=gtk_text_new(NULL,NULL);
-	gtk_signal_connect(GTK_OBJECT(text),"style-set",signal_style_set,NULL);
+#endif
+	gtk_signal_connect(GTK_OBJECT(text),"style-set",
+									GTK_SIGNAL_FUNC(signal_style_set),NULL);
 	/* ľܥå/ϥɥܥå */
 	vbox=gtk_vbox_new(FALSE,0);
 	handle=gtk_handle_box_new();
@@ -364,25 +436,34 @@ int main(int argc,char *argv[])
 
 	/* ˵ưƤ뤫å */
 	if (!second) {
-		for (i=1;i<argc;i++) {
-			if (arg_files[i-1]) {
-				arg=misc_get_full_path(argv[i]);
-				leng=strlen(arg)*sizeof(gchar);
-				arg_text=g_realloc(arg_text,length+leng+sizeof(gchar));
-				memcpy(arg_text+length,arg,leng);
-				g_free(arg);
-			} else {
-				leng=strlen(argv[i])*sizeof(gchar);
-				arg_text=g_realloc(arg_text,length+leng+sizeof(gchar));
-				memcpy(arg_text+length,argv[i],leng);
-			}
-			length+=leng;
-			arg_text[length++]='\n';
-		}
-		if (length>0) {
-			arg_text[length-1]='\0';
-			gtk_selection_owner_set(window,atom_videomaid1,GDK_CURRENT_TIME);
-		}
+		if (argc > 1)
+			for (i = 0; i < argc; i++)
+			  {
+				if (arg_files[i])
+				  {
+					arg = misc_get_full_path (argv[i]);
+					leng = g_strlen (arg) * sizeof (gchar);
+					arg_text = g_realloc (arg_text,
+											length + leng + sizeof (gchar));
+					g_memmove (arg_text + length, arg, leng);
+					g_free (arg);
+				  }
+				else
+				  {
+					leng = g_strlen (argv[i]) * sizeof (gchar);
+					arg_text = g_realloc (arg_text,
+											length + leng + sizeof (gchar));
+					g_memmove (arg_text + length, argv[i], leng);
+				  }
+				length += leng;
+				arg_text[length++] = '\n';
+			  }
+		if (length > 0)
+		  {
+			arg_text[length - 1] = '\0';
+			gtk_selection_owner_set (window, atom_videomaid1,
+															GDK_CURRENT_TIME);
+		  }
 		gtk_selection_convert(window,atom_videomaid0,
 								GDK_SELECTION_TYPE_STRING,GDK_CURRENT_TIME);
 		gdk_threads_enter();
@@ -425,31 +506,39 @@ int main(int argc,char *argv[])
 	system_color[2]=style->dark[0];
 	system_color[3]=style->bg[3];
 	/* ƥե */
-	system_font=gdk_font_ref(style->font);
+	system_font=gdk_font_ref(gtk_style_get_font(style));
 	for (i=0;i<256;i++) {
 		gdk_text_extents(system_font,(gchar *)&i,1,
 												NULL,NULL,NULL,&ascent,NULL);
 		if (system_font_ascent<ascent)
 			system_font_ascent=ascent;
-		height=gdk_char_height(system_font,i);
+		height = gdk_char_height (system_font, (gchar)i);
 		if (system_font_height<height)
 			system_font_height=height;
 	}
 
 	/* ɥ */
 	for (i=1;i<argc;i++)
-		if (arg_files[i-1]) {
+		if (arg_files[i]) {
 			file_open=g_malloc(sizeof(FileOpen));
 			file_open->file=g_strdup(argv[i]);
 			file_open->merge=file_merge;
+#ifdef USE_GTK2
+			g_thread_create((GThreadFunc)file_open_edit,file_open,TRUE,NULL);
+#else
 			pthread_create(&id,NULL,(void *)file_open_edit,file_open);
 			pthread_detach(id);
+#endif
 			count++;
 		}
 	g_free(arg_files);
 	if (newfile && count<=0) {
+#ifdef USE_GTK2
+		g_thread_create((GThreadFunc)file_open_edit,NULL,TRUE,NULL);
+#else
 		pthread_create(&id,NULL,(void *)file_open_edit,NULL);
 		pthread_detach(id);
+#endif
 	}
 
 	gdk_threads_enter();
@@ -460,7 +549,7 @@ int main(int argc,char *argv[])
 	gdk_bitmap_unref(bitmap1);
 
 	/* 쥸ȥ˽񤭹 */
-	if ((profile=profile_open(".vmaid"))!=NULL) {
+	if ((profile=profile_open(NULL))!=NULL) {
 		profile_set_value(profile,"vmaid","history",
 				(gpointer)&history,sizeof(gint),PROFILE_VALUE_TYPE_INT);
 		profile_set_value(profile,"vmaid","newfile",
@@ -483,17 +572,17 @@ int main(int argc,char *argv[])
 				(gpointer)&default_rate,sizeof(guint),PROFILE_VALUE_TYPE_INT);
 		if (open_path!=NULL)
 			profile_set_value(profile,"vmaid","openpath",(gpointer)open_path,
-										(strlen(open_path)+1)*sizeof(gchar),
+										(g_strlen(open_path)+1)*sizeof(gchar),
 													PROFILE_VALUE_TYPE_STRING);
 		if (save_path!=NULL)
 			profile_set_value(profile,"vmaid","savepath",(gpointer)save_path,
-										(strlen(save_path)+1)*sizeof(gchar),
+										(g_strlen(save_path)+1)*sizeof(gchar),
 													PROFILE_VALUE_TYPE_STRING);
 		/* ե */
 		for (i=0;i<history && history_files[i]!=NULL;i++) {
 			key=g_strdup_printf("history%d",i);
 			profile_set_value(profile,"vmaid",key,(gpointer)history_files[i],
-									(strlen(history_files[i])+1)*sizeof(gchar),
+								(g_strlen(history_files[i])+1)*sizeof(gchar),
 													PROFILE_VALUE_TYPE_STRING);
 			g_free(key);
 		}
@@ -511,5 +600,6 @@ int main(int argc,char *argv[])
 	g_free(save_path);
 	for (i=0;i<history;i++)
 		g_free(history_files[i]);
+	gtk_exit(0);
 	return 0;
 }
